
package org.one.stone.soup.wiki.client;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;

import org.one.stone.soup.file.FileHelper;
import org.one.stone.soup.stringhelper.StringArrayHelper;

import sun.misc.BASE64Encoder;

public final class WikiClient {

	public static final String SANDBOX = "Sandbox";

	public static final String FORM_BOUNDARY = "----puoSenotSenO";
	
	private static String PAGE_UPLOAD_URL = "/OpenForum/Actions/Update";
	private static String FILE_UPLOAD_URL = "/OpenForum/Actions/Attach?page=";
	private static String PAGES_LIST_URL = "/OpenForum/Actions/GetPagesList?pageName=anyPage";
	
	private String url;
	private String userId;
	private String password;

	public WikiClient(String url,String userId,String password)
	{
		this.url = url;
		this.userId = userId;
		this.password = password;
	}

	
	public String getUrl() {
		return url;
	}

	public boolean getBoolean(String page)
	{
		String urlString = url+"/"+page;

		URL url = null;
	
		try{
			url = new URL(urlString);
		}
		catch(MalformedURLException mue)
		{
			mue.printStackTrace();
			return false;
		}
		
		try{
			String result = new String( FileHelper.readFile( url.openStream() ) );
			
			if(result.equals("OK") || result.equals("true"))
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return false;
		}
	}
	
	public String[] getPages()
	{
		String urlString = url+PAGES_LIST_URL;

		URL url = null;
	
		try{
			url = new URL(urlString);
		}
		catch(MalformedURLException mue)
		{
			mue.printStackTrace();
			return null;
		}
		
		try{
			String htmlData = new String(FileHelper.readFile( url.openStream() ));

			String[] pagesArray = StringArrayHelper.parseFields(htmlData,',');
			
			return StringArrayHelper.sort(pagesArray,StringArrayHelper.ASCENDING);
			//return pagesArray;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return null;
		}
	}
	
	public boolean uploadFile(String pageName,String fileName,InputStream fileStream)
	{
		if(pageName==null)
		{
			pageName=SANDBOX;
		}
		
		String urlString = url+FILE_UPLOAD_URL+pageName+"&fileName="+fileName;

		URL url = null;
	
		try{
			url = new URL(urlString);
		}
		catch(MalformedURLException mue)
		{
			mue.printStackTrace();
			return false;
		}

		try{
			URLConnection connection = url.openConnection();
	
			connection.setDoInput(true);
			connection.setDoOutput(true);
			connection.setUseCaches(false);
	
			connection.setRequestProperty( "Content-Type","multipart/form-data; boundary="+FORM_BOUNDARY );
			
			if(userId!=null)
			{
				String auth = new BASE64Encoder().encode( (userId+":"+password).getBytes() );
				connection.setRequestProperty( "Authorization","Basic="+auth );
			}
			
			connection.setRequestProperty( "User-Agent","Wiki Poster");

			OutputStream outStream = connection.getOutputStream();
			PrintWriter out = new PrintWriter( outStream );
	
			out.println("--"+FORM_BOUNDARY);
			out.println("Content-Disposition: form-data; name=\"filename\"; filename=\""+URLEncoder.encode(fileName)+"\"");
			out.println("Content-Type: application/octet-stream");
			out.println();
			out.flush();

			BufferedInputStream fileBuffer = new BufferedInputStream(fileStream);
			byte[] dataBuffer = new byte[1000];
			int size = fileBuffer.read(dataBuffer);
		
			while(size>0)
			{
				outStream.write( dataBuffer,0,size );
				size = fileBuffer.read(dataBuffer);
				outStream.flush();
			}
			fileStream.close();
	
			out.print("\r\n--"+FORM_BOUNDARY+"--");
			out.flush();
	
			String result = new String(FileHelper.readFile( connection.getInputStream() ));
			System.out.println("Result: "+result);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return false;
		}
		
		return true;
	}
	
	public boolean uploadFile(String pageName,String fileName,WikiFileWriter fileWriter)
	{
		if(pageName==null)
		{
			pageName=SANDBOX;
		}
		
		String urlString = url+FILE_UPLOAD_URL+pageName;

		URL url = null;
	
		try{
			url = new URL(urlString);
		}
		catch(MalformedURLException mue)
		{
			mue.printStackTrace();
			return false;
		}

		try{
			URLConnection connection = url.openConnection();
	
			connection.setDoInput(true);
			connection.setDoOutput(true);
			connection.setUseCaches(false);
	
			connection.setRequestProperty( "Content-Type","multipart/form-data; boundary="+FORM_BOUNDARY );
			if(userId!=null)
			{
				String auth = new BASE64Encoder().encode( (userId+":"+password).getBytes() );
				connection.setRequestProperty( "Authorization","Basic="+auth );
			}
			
			connection.setRequestProperty( "User-Agent","Wiki Poster");

			OutputStream outStream = connection.getOutputStream();
			PrintWriter out = new PrintWriter( outStream );
	
			out.println("--"+FORM_BOUNDARY);
			out.println("Content-Disposition: form-data; name=\"filename\"; filename=\""+URLEncoder.encode(fileName)+"\"");
			out.println("Content-Type: application/octet-stream");
			out.println();
			out.flush();
			
			fileWriter.writeFileTo( outStream );
	
			out.print("\r\n--"+FORM_BOUNDARY+"---\n");
			out.flush();
	
			InputStream inStream = connection.getInputStream();
			String result = new String(FileHelper.readFile( inStream ));
			//System.out.println("Result: "+result);
			
			//out.print("\n");
			//out.flush();			
			
			//outStream.flush();
			//outStream.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return false;
		}
		
		return true;
	}
	
	public boolean uploadPage(String pageName,String pageData)
	{
		if(pageName==null)
		{
			pageName=SANDBOX;
		}
		
		String urlString = PAGE_UPLOAD_URL;

		URL url = null;

		try{
			url = new URL(urlString);
		}
		catch(MalformedURLException mue)
		{
			mue.printStackTrace();
			return false;
		}
		
		try{
			URLConnection connection = url.openConnection();
	
			connection.setDoInput(true);
			connection.setDoOutput(true);
			connection.setUseCaches(false);
	
			connection.setRequestProperty( "Content-Type","application/x-www-form-urlencoded" );
			connection.setRequestProperty( "User-Agent","Wiki Poster");
	
			PrintWriter out = new PrintWriter( connection.getOutputStream() );
	
			StringBuffer dataString = new StringBuffer();

			dataString.append( "pageName="+URLEncoder.encode( pageName ) );
			dataString.append( "&source="+URLEncoder.encode( pageData ) );
			
			out.print( dataString.toString() );
			out.flush();
	
			String result = new String(FileHelper.readFile( connection.getInputStream() ));
			System.out.println("Result: "+result);
		}
		catch(Exception e)
		{
			return false;
		}
		return true;
	}


	public void setPassword(String password) {
		this.password = password;
	}


	public void setUserId(String userId) {
		this.userId = userId;
	}
}
