package org.one.stone.soup.wiki.upload.applet;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.URL;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.swing.JProgressBar;

public class Uploader implements Runnable{

	private Vector uploadQueue = null;
	private boolean running = false;
	private Thread thread;
	private String authentication;
	
	private class Upload
	{
		public Upload(URL url,String fileName)
		{
			this.url = url;
			this.fileName = fileName;
		}
		public URL url;
		public String fileName;
	}
	
	private JProgressBar progressBar;
	public String progressMessage;
	public int progressPercentage;

	private static Uploader uploader = new Uploader();
	
	public static final Uploader getUploader()
	{
		if(uploader==null)
		{
			uploader = new Uploader();
		}
		
		return uploader;
	}
	
	private Uploader()
	{
		System.out.println("New Uploader created "+this);
		progressMessage = "Drop Attachments Here";
		progressPercentage = 100;
		
		uploadQueue = new Vector();
	}
	
	private void startQueueThread()
	{
		thread = new Thread(this,"Uploader");
		thread.setDaemon(true);
		thread.start();
		
		System.out.println("New queue thread started "+thread);
	}
	
	public void upload(URL url,String fileName)
	{
		Upload upload = new Upload(url,fileName);
		
		if(running==false)
		{
			startQueueThread();
		}
		
		uploadQueue.addElement(upload);
	}
	
	
	public void run()
	{
		running = true;
		try{
			while(true)
			{
				if(uploadQueue.size()>0)
				{
					Upload upload = null;
					try{
						upload = (Upload)uploadQueue.elementAt(0);
						
						doUpload( upload );
						uploadQueue.remove(upload);
					}
					catch(Exception e)
					{
						e.printStackTrace();
						uploadQueue.remove(upload);
						JOptionPane.showMessageDialog(progressBar,"Failed to upload "+upload.fileName+" to page "+upload.url+"\n"+e );
					}
				}
				else
				{
					setProgress(100);
					setProgress("Drop Attachments Here");
				}
			}
		}
		finally
		{
			running = false;
		}
	}
	
	private void doUpload( Upload upload ) throws Exception
	{	   		
	   File file = new File(upload.fileName);
	   String fileName = new File(upload.fileName).getName();
	 
		setProgress("Uploading "+fileName);
		
	   URL root = upload.url;
	   String pageName = root.toExternalForm();
	   pageName = pageName.substring( 7 );
	   pageName = pageName.substring( pageName.indexOf("/") );	   
	   
	   Socket connection = null;
	   InputStream iStream = null;
	   
	   try{		   
			WikiURLConnection wiki = new WikiURLConnection( upload.url,authentication );
			connection = wiki.openPutConnection(pageName,fileName,file.length());		   
		   
			OutputStream oStream = connection.getOutputStream();
			iStream = new FileInputStream(file);
			int total = (int)file.length();
		   
			setProgress(0);
		   
			byte[] chunk = new byte[5000];
			int size = iStream.read(chunk);
			int count = 0;
			while(size!=-1)
			{
				   oStream.write(chunk,0,size);
				   count+=size;
				   size = iStream.read(chunk);
				   
				   setProgress(count*100/total);
			}
			iStream.close();
			oStream.flush();
			connection.close();
	   }
	   catch(Exception e)
	   {
		   throw e;
	   }
	   finally
	   {
		   try{
			   iStream.close();
			   connection.close();
		   }
		   catch(Exception e)
		   {}
	   }
	}

	public void setProgressBar(JProgressBar progressBar) {
		this.progressBar = progressBar;
		
		progressBar.setStringPainted(true);
		progressBar.setString(progressMessage);
		progressBar.setMinimum(0);
		progressBar.setMaximum(100);
	}
	
	private void setProgress(String message)
	{
		progressMessage = message;
		if(progressBar!=null)
		{
			progressBar.setString(message);
		}
	}
	
	private void setProgress(int percentage)
	{
		progressPercentage = percentage;
		if(progressBar!=null)
		{
			progressBar.setValue(percentage);
		}
	}
	
	public void setAuthentication(String authentication)
	{
		this.authentication = authentication;
	}
}
