package org.one.stone.soup.xml.file;

import java.io.File;

import org.one.stone.soup.file.FileHelper;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlBuilder;
import org.one.stone.soup.xml.helper.XmlCloner;
import org.one.stone.soup.xml.helper.XmlPathSearch;
import org.one.stone.soup.xml.stream.XmlLoader;

public class FilesToXml {
	
	public static void main(String[] args)
	{
		if(args.length==0)
		{
			System.out.println("Usage: FilesToXml [createFor directory xmlFileName|compare oldXmlFile newXmlFile differencesXmlFile]");
			System.exit(1);
		}
		 if(args[0].equals("createFor") && args.length==3)
		 {
			 String directory = args[1];
			 String xmlFileName = args[2];
			 
			 File folder = new File(directory);
			 
			 System.out.println("Creating Xml Manifest for "+folder.getAbsolutePath());
			 XmlElement result = directoryToXml( folder.getAbsolutePath().length(),folder,true );
			 FileHelper.buildFile( xmlFileName,result.toXml() );
		 }
		 else if(args[0].equals("compare") && args.length==4)
		 {
			 String oldXmlFile = args[1];
			 String newXmlFile = args[2];
			 String differencesXml = args[3];

			 XmlElement oldXml = XmlLoader.load(oldXmlFile);
			 XmlElement newXml = XmlLoader.load(newXmlFile);
			 setDifferences(oldXml,newXml);
			 FileHelper.buildFile( differencesXml,oldXml.toXml() );

		 }
		 else
		 {
				System.out.println("Usage: FilesToXml [createFor directory xmlFileName|compare oldXmlFile newXmlFile differencesXmlFile]");
				System.exit(1);			 
		 }
	}
	
	public static final String STATE_UNCHANGED = "unchanged";
	public static final String STATE_CHANGED = "changed";
	public static final String STATE_DELETED = "deleted";
	public static final String STATE_NEW = "new";
	
	private FilesToXml(){}
	
	public static XmlElement directoryToXml(File directory,boolean includeMD5)
	{
		return directoryToXml(0,directory,includeMD5);
	}
	
	public static XmlElement directoryToXml(int rootOffest,File directory,boolean includeMD5)
	{
		if( !directory.exists() )
		{
			return null;
		}
		else if(directory.isDirectory()==false)
		{
			return fileToXml( rootOffest,directory,includeMD5 );
		}
		
		XmlElement xDirectory = new XmlElement("directory");
		xDirectory.addAttribute( "name",directory.getName() );
		
		xDirectory.addAttribute( "src",directory.getAbsolutePath().replace('\\','/').substring(rootOffest) );
		xDirectory.addAttribute( "timestamp",""+directory.lastModified() );
		
		File[] files = directory.listFiles();
		xDirectory.addAttribute( "files",""+files.length );
		
		for(int loop=0;loop<files.length;loop++)
		{
			if(files[loop].isDirectory())
			{
				xDirectory.addChild( directoryToXml(rootOffest,files[loop],includeMD5) );
			}
			else
			{
				xDirectory.addChild( fileToXml(rootOffest,files[loop],includeMD5) );
			}
		}
		
		return xDirectory;
	}
	
	public static XmlElement fileToXml(File file,boolean includeMD5)
	{
		return fileToXml(0,file,includeMD5);
	}	
	
	public static XmlElement fileToXml(int rootOffest,File file,boolean includeMD5)
	{
		if( !file.exists() || file.isDirectory() )
		{
			return null;
		}
		
		XmlElement xFile = new XmlElement("file");
		xFile.addAttribute( "name",file.getName() );
		xFile.addAttribute( "src",file.getAbsolutePath().replace('\\','/').substring(rootOffest) );
		xFile.addAttribute( "timestamp",""+file.lastModified() );
		xFile.addAttribute( "size",""+file.length() );
		if(includeMD5==true)
		{
			try{
				xFile.addAttribute( "md5",FileHelper.createMD5(file.getAbsolutePath()) );
			}
			catch(Exception e)
			{
			}
		}
		
		return xFile;
	}
	
	public static boolean setDifferences(XmlElement original,XmlElement current)
	{
		boolean hasDifferences = false;
		
		if(original.getName().equals("file"))
		{
			if(
				original.getAttributeValueByName("md5")!=null &&
				current.getAttributeValueByName("md5")!=null
			)
			{
				if( original.getAttributeValueByName("md5").equals( current.getAttributeValueByName("md5") ) )
				{
					XmlBuilder.setAttribute(original,"state",STATE_UNCHANGED);
					return false;
				}
				else
				{
					XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
					return true;
				}
			}
					
			if( !original.getAttributeValueByName("timestamp").equals(current.getAttributeValueByName("timestamp")) )
			{
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
				return true;
			}
			if( !original.getAttributeValueByName("size").equals(current.getAttributeValueByName("size")) )
			{
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
				return true;
			}
			XmlBuilder.setAttribute(original,"state",STATE_UNCHANGED);
			return false;
		}
		else
		{
			if( !original.getAttributeValueByName("timestamp").equals(current.getAttributeValueByName("timestamp")) )
			{
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
			}
			if( !original.getAttributeValueByName("files").equals(current.getAttributeValueByName("files")) )
			{
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
			}			
		}
		
		for(int loop=0;loop<original.getElementCount();loop++)
		{
			XmlElement testElement = original.getElementByIndex(loop);
			
			XmlPathSearch search = new XmlPathSearch(current);
			XmlElement found = search.findElement( current.getName()+"/"+testElement.getName()+"[@name='"+testElement.getAttributeValueByName("name")+"']" );
			
			if(found==null)
			{
				hasDifferences=true;
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
				XmlBuilder.setAttribute(testElement,"state",STATE_DELETED);
			}
			else
			{
				if( setDifferences(testElement,found)==true )
				{
					hasDifferences = true;
					XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
				}
			}
		}
		
		for(int loop=0;loop<current.getElementCount();loop++)
		{
			XmlElement testElement = current.getElementByIndex(loop);
			
			XmlPathSearch search = new XmlPathSearch(original);
			XmlElement found = search.findElement( original.getName()+"/"+testElement.getName()+"[@name='"+testElement.getAttributeValueByName("name")+"']" );
			
			if(found==null)
			{
				hasDifferences=true;
				XmlBuilder.setAttribute(original,"state",STATE_CHANGED);
				XmlElement childElement = XmlCloner.getClone(testElement,false);
				original.addChild(childElement);
				XmlBuilder.setAttribute(childElement,"state",STATE_NEW);
				
				setDifferences(childElement,testElement);
			}
		}		
		
		return hasDifferences;
	}
}
