package org.one.stone.soup.xml.helper;

import java.util.*;
import java.lang.reflect.*;

import org.one.stone.soup.xml.*;
import org.one.stone.soup.stringhelper.*;

/*
 * Wet-Wired.com Xml Helpers Version Date 25-Aug-2004
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.co.uk/pages/licence-os.html
 */

/**
 * @author nikcross
 *
 */

public class XmlHelper {
private XmlHelper() {
	super();
}
public static Object getElementAsClassInstance(XmlElement element, String name)
throws
	ClassNotFoundException,
	InstantiationException,
	IllegalAccessException
{

	String className = element.getElementByName(name).getValue(0);

	Class clazz = Class.forName(className);

	Object instance = clazz.newInstance();

	return instance;
}
public static Object getElementAsClassInstance(XmlElement element, String name, String args[])
throws
	ClassNotFoundException,
	InstantiationException,
	NoSuchMethodException,
	IllegalAccessException,
	InvocationTargetException
{

	String className = element.getElementByName(name).getValue(0);

	Class clazz = Class.forName(className);

	Object instance = clazz.getConstructor(
		new Class[]{ Class.forName("[Ljava.lang.String;") }
			).newInstance(
					new Object[]{ args }
				);

	return instance;
}
public static int getElementAsInt(XmlElement element, String name) {
	return Integer.parseInt( element.getElementByName(name).getValue(0) );
}
public static String getElementAsString(XmlElement element, String name) {
	return element.getElementByName(name).getValue(0);
}
public static String[] getElementAsStringArray(XmlElement element, String name) {
	Vector data = (Vector)element.getElementByName(name).getData(0);

	return StringArrayHelper.vectorToStringArray(data);
}

public static Hashtable getElementAsHashtable(XmlElement element,String keyAttribute) {
	Hashtable table = new Hashtable();
	
	for(int loop=0;loop<element.getElementCount();loop++)
	{
		XmlElement item = element.getElementByIndex(loop);
		table.put(item.getAttributeValueByName(keyAttribute),item.getValue());
	}

	return table;
}

public static Hashtable getElementAsHashtable(XmlElement element,String keyAttribute,String valueAttribute) {
	Hashtable table = new Hashtable();
	
	for(int loop=0;loop<element.getElementCount();loop++)
	{
		XmlElement item = element.getElementByIndex(loop);
		String key=item.getAttributeValueByName(keyAttribute);
		String value=item.getAttributeValueByName(valueAttribute);
		if(key==null || value==null)
		{
			continue;
		}
		table.put(key,value);
	}

	return table;
}

public static Hashtable getElementAsHashtable(XmlElement element) {
	Hashtable table = new Hashtable();
	
	for(int loop=0;loop<element.getElementCount();loop++)
	{
		XmlElement item = element.getElementByIndex(loop);
		table.put(item.getName(),item.getValue());
	}

	return table;
}

public static XmlElement getHashtableAsElement(String name,Hashtable table) {
	XmlElement element = new XmlElement(name);
	
	Enumeration enumeration = table.keys();
	while(enumeration.hasMoreElements())
	{
		String key = (String)enumeration.nextElement();
		XmlElement item = element.addChild("item");
		item.addAttribute("key",key);
		item.addDataItem( table.get(key) );
	}

	return element;
}
/**
 *
 * @param item org.one.stone.soup.xml.XmlElement
 * @param set org.one.stone.soup.xml.XmlElement
 */
public static boolean isInSet(String itemName, XmlElement set)
{
	if(set!=null)
	{
			if(set.getAttributeValueByName("type").equals("exclude"))
			{
				if( set.getElementByName( itemName)!=null )
				{
					return false;
				}
			}
			else if(set.getAttributeValueByName("type").equals("include"))
			{
				if( set.getElementByName( itemName)==null )
				{
					return false;
				}
			}
			else
			{
				return false;
			}
	}

	return true;
}
/**
 *
 * @param item org.one.stone.soup.xml.XmlElement
 * @param set org.one.stone.soup.xml.XmlElement
 */
public static boolean isInSet(XmlElement item, XmlElement set)
{
	if(set!=null)
	{
			if(set.getAttributeValueByName("type").equals("exclude"))
			{
				if( set.getElementByName( item.getName())!=null )
				{
					return false;
				}
			}
			else if(set.getAttributeValueByName("type").equals("include"))
			{
				if( set.getElementByName( item.getName())==null )
				{
					return false;
				}
			}
			else
			{
				return false;
			}
	}

	return true;
}
/**
 *
 */
public static int getIndexOf(XmlElement source,XmlElement child)
{
	for(int loop=0;loop<source.getElementCount();loop++)
	{
		XmlElement node = source.getElementByIndex(loop);
		if(node==child)
		{
			return loop;
		}
	}

	return -1;
}
/**
 *
 */
public static XmlElement getParent(XmlElement source,XmlElement child)
{

	for(int loop=0;loop<source.getElementCount();loop++)
	{
		XmlElement node = source.getElementByIndex(loop);
		if(node==null)
		{
			continue;
		}

/*		if(node instanceof XmlElementWrapper && ((XmlElementWrapper)node).getContents()==child )
		{
			return source;
		}*/
		if(node==child)
		{
			return source;
		}

		XmlElement found = getParent(node,child);
		if(found!=null)
		{
			return found;
		}
	}

	return null;
}
public static String getIndexPath(XmlElement root,XmlElement node)
{
	XmlElement parent = getParent(root,node);
	if(parent==null)
	{
		return "";
	}

	int index = -1;
	String indexString = "";
	while(parent!=null)
	{
		index = getIndexOf(parent,node);
		indexString = index+"."+indexString;

		node = parent;
		parent = getParent(root,node);
	}
	return indexString.substring(0,indexString.length()-1);
}
public static XmlElement getElementByIndexPath(XmlElement root,String indexPath)
{
	String[] path = StringArrayHelper.parseFields(indexPath,'.');
	int[] intPath = new int[path.length];
	for(int loop=0;loop<intPath.length;loop++)
	{
		intPath[loop] = Integer.parseInt( path[loop] );
	}
	
	return getElementByIndexPath(root,intPath);
}
public static XmlElement getElementByIndexPath(XmlElement root,int[] indexPath)
{
	if(indexPath.length==0)
	{
		return null;
	}

	XmlElement result = root;
	for(int loop=0;loop<indexPath.length;loop++)
	{
		int index = indexPath[loop];
		if(index<0 || result.getElementCount()<index)
		{
			return null;
		}
		else
		{
			result = result.getElementByIndex(index);
		}
		if(result==null)
		{
			return null;
		}
	}

	return result;
}
}
